package com.jqtx.windows.common.utils;


import com.jqtx.windows.common.exception.BizException;
import com.jqtx.windows.infrastructure.enums.ExceptionCodeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Component;
import java.io.Serializable;
import java.util.concurrent.TimeUnit;

@Component
public class RedisUtils {

    @Autowired
    private RedisTemplate redisTemplate;


    /**
     *
     * @param key
     * @param value
     * @return
     */
    public void urlLimit(String key) {
        if(!this.setnx(key,null,10)){
            throw new BizException(ExceptionCodeEnum.HTTP_FAST_ERROR);
        }
    }

    /**
     * @param key
     * @param value
     * @return
     */
    public boolean setnx(String key, String value) {
        ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
        if (operations.setIfAbsent(key, value)) {//对应setnx命令
            return true;
        }
        return false;
    }
    /**
     * setnx
     * @param key
     * @param value
     * @return
     */
    public boolean setnx(String key, Object value, long second) {
        ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
        if (operations.setIfAbsent(key, value)) {//对应setnx命令
            //可以成功设置,也就是key不存在
            redisTemplate.expire(key, second, TimeUnit.SECONDS);
            return true;
        }
        return false;
    }

    /**
     * 写入缓存
     *
     * @param key
     * @param value
     * @return
     */
    public void set(final String key, Object value) {
        ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
        operations.set(key, value);
    }

    /**
     * 写入缓存设置时效时间
     *
     * @param key
     * @param value
     * @return
     */
    public void set(final String key, Object value, long expireTime) {
        ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
        operations.set(key, value);
        redisTemplate.expire(key, expireTime, TimeUnit.SECONDS);

    }

    /**
     *
     *自增
     * @param key
     * @param value
     * @return
     */
    public long increment(final String key, long value) {
        ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
        return operations.increment(key, value);
    }

    /**
     * 删除对应的value
     *
     * @param key
     */
    public void remove(final String key) {
        if (redisTemplate.hasKey(key)) {
            redisTemplate.delete(key);
        }
    }
    /**
     * 判断缓存中是否有对应的value
     *
     * @param key
     * @return
     */
    public boolean exists(final String key) {
        return redisTemplate.hasKey(key);
    }

    /**
     * 读取缓存
     *
     * @param key
     * @return
     */
    public Object get(final String key) {
        ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
        return operations.get(key);
    }
}

